
#include "gba.h"
#include "backgrounds.h"
#include "screenmode.h"

void EnableBackground(Bg* bg)
{
	u16 temp;

	bg->tileData = (u16*)CharBaseBlock(bg->charBaseBlock);
	bg->mapData = (u16*)ScreenBaseBlock(bg->screenBaseBlock);
	temp = bg->size | (bg->charBaseBlock<<CHAR_SHIFT) | (bg->screenBaseBlock<<SCREEN_SHIFT) 
		| bg->colorMode | bg->mosiac;

	switch(bg->number)
	{
	case 0:
		{
			REG_BG0CNT = temp;
			REG_DISPCNT |= BG0_ENABLE;
		}break;
	case 1:
		{
			REG_BG1CNT = temp;
			REG_DISPCNT |= BG1_ENABLE;
		}break;
	case 2:
		{
			REG_BG2CNT = temp;
			REG_DISPCNT |= BG2_ENABLE;
		}break;
	case 3:
		{
			REG_BG3CNT = temp;
			REG_DISPCNT |= BG3_ENABLE;
		}break;
	
	default:break;
	
	}
}

void UpdateBackground(Bg* bg)
{
	switch(bg->number)
	{
	case 0:
		REG_BG0HOFS = bg->x_scroll;
		REG_BG0VOFS = bg->y_scroll;
		break;
	case 1:
		REG_BG1HOFS = bg->x_scroll;
		REG_BG1VOFS = bg->y_scroll;
		break;
	case 2:
		if(!(REG_DISPCNT & MODE_0))//it is a rot background
		{
		  REG_BG2X = bg->DX;
		  REG_BG2Y = bg->DY;

		  REG_BG2PA = bg->PA;
		  REG_BG2PB = bg->PB;
		  REG_BG2PC = bg->PC;
		  REG_BG2PD = bg->PD;
		}
		else  //it is a text background
		{
		  REG_BG2HOFS = bg->x_scroll;
		  REG_BG2VOFS = bg->y_scroll;
		}
		break;
	case 3:
		if(!(REG_DISPCNT & MODE_0))//it is a rot background
		{
			REG_BG3X = bg->DX;
			REG_BG3Y = bg->DY;
			
			REG_BG3PA = bg->PA;
			REG_BG3PB = bg->PB;
			REG_BG3PC = bg->PC;
			REG_BG3PD = bg->PD;
		}
		else //it is a text background
		{
			REG_BG3HOFS = bg->x_scroll;
			REG_BG3VOFS = bg->y_scroll;
		}
		break;
	default: break;
	}
}

void ClearScreen(Bg * bg) {
  u32 loop;
  for(loop = 0; loop < 1024; loop++) //64x64 tiles /2 because 16 bit copy
    bg->mapData[loop] = -1;
}
void SetBackground(Bg * bg) {
	u16 temp = bg->size | (bg->charBaseBlock<<CHAR_SHIFT) | (bg->screenBaseBlock<<SCREEN_SHIFT) 
		| bg->colorMode | bg->mosiac;

	bg->tileData = (u16*)CharBaseBlock(bg->charBaseBlock);
	bg->mapData = (u16*)ScreenBaseBlock(bg->screenBaseBlock);
	switch(bg->number)
	{
		case 0:
			{
				REG_BG0CNT = temp;
				REG_DISPCNT |= BG0_ENABLE;
			}break;
		case 1:
			{
				REG_BG1CNT = temp;
				REG_DISPCNT |= BG1_ENABLE;
			}break;
		case 2:
			{
				REG_BG2CNT = temp;
				REG_DISPCNT |= BG2_ENABLE;
			}break;
		case 3:
			{
				REG_BG3CNT = temp;
				REG_DISPCNT |= BG3_ENABLE;
			}break;

		default:break;

	}
}


void initBackground(Bg * bg, int number,
                    int charBlock, int screenBlock, 
                    int priority) 
{
  bg->number = number;
  bg->charBaseBlock = charBlock;
  bg->screenBaseBlock = screenBlock;
  bg->colorMode = BG_COLOR_16;
  bg->size = TEXTBG_SIZE_256x256; 
  // not really mosaic, but priority setting. This could be called 
  // "flags" really, as it's all just ORed together with the size and colour.
  bg->mosiac = priority;
 // bg->y_scroll = (24) &0xff;
  EnableBackground(bg);
  UpdateBackground(bg);

  
}

