/* qmath
  
  implementation of maths functions to save using the gcc lib ones
  these are more GBA oriented. 
  
  */

#include "qmath.h"  
  
/*    qarctan
    calculate arctan of a number using a 
    crappy approximation, return value in degrees
    error in approx <=0.005 radians (.285 degrees!) 
    the rounding errors are larger
*/
u16 qarctan(u16 opposite, u16 adjacent) {
  // undefined, return 0
  if (adjacent == 0)
    return 0;
  FIXED a; // our arctan value
  
  FIXED x = (FIXED)((opposite<<8)/adjacent);
  FIXED pi_2 = 402;
  
  FIXED factor = 72;
  FIXED x2 = (FIXED)((x * x)>>8); // shift down 8 for multiply
  // |x|<=1 use one approximation
  // atan(x) = x/(1+0.28*x^2)
  if (x <= 256) {
    FIXED denom = ((256) + ((factor * x2)>>8)); // shift down 8 for multiply
    a = (FIXED)((x<<8)/denom); // shift up 8 for divide
  } else {
    // |x|>1 use another approximation
    // arctan(x) = (pi/2 - x/(x^2+0.28))
    FIXED denom = x2+factor;
    a = (FIXED)((pi_2) - ((x<<8)/denom));
  }
  // finally, return the result in degrees
  a = a * 57; //57 = 180/pi
  return (u16)(a>>8); // shift down!
}

/*
  Random number routine as posted to the gbadev list 
  by John Seghers <johnse at spiritdancers dot org>
  
  Uses linear congruential, this formula is attributed to Ahrens & Dieter
  LCG(2^32, 663608941, 0, 1)
  
  UltraEdit reg expression for replacing rand()%anything with GetRand(anything)
          find: rand()?^([0-9,a-Z,^[^]^.^_]+^)
  replace with: GetRand(^1)
*/

u32 m_nRandom = 14071977; // default seed
void SeedRand(u32 seed) {
  m_nRandom = seed;
}
void ChurnRand(void) {
  m_nRandom *= 663608941; // churn the random number
}


u32 GetRand(u32 nMax) {
  unsigned long long nResult = nMax;
  nResult = nResult * m_nRandom;
  //m_nRandom *= 663608941; // churn the random number
  ChurnRand();
  return (u32)(nResult >> 32);
}
