// saving handler
#include "saving.h"

#define MAX_KEY_LENGTH 10

#if 0
void SaveByte(unsigned short nOffset, unsigned char cValue);
unsigned char LoadByte(unsigned short nOffset);
void UnencryptSaveData(unsigned char* pDataSrc, unsigned short nDataLength, unsigned char* pDataDest);
void EncryptSaveData(unsigned char* pDataSrc, unsigned short nDataLength, unsigned char* pDataDest);

const static char cEncryptionKey[10] = "O5v64Ez2P+";


unsigned short GetLoadDataSize(char* pKey) {
  // 1st check the key is from our game...
  unsigned short i;
  for (i = 0; i < 3; i++) {
    if (LoadByte(i) != pKey[i]) {
      return 0;
    }
  }
  // is our game! how much ram to load the data?
  unsigned short nSize = LoadByte(3) + LoadByte(4)*256;
  // byte 4,5 are the data size
  
  if (nSize > (1<<15)) {
    // too much data..
    return 0;
  }
  return nSize+1;
}

void LoadData(unsigned char * pDataDest, char* pKey) {
  // load the data which has the given pKey
  // pDataDest pointer is already alloc'd
  
  // the save structure is:
  // byte 0, 1, 2 - Key, e.g. "CYC" for cyclone
  // byte 3, 4 - size of save data, not including key or these bytes (byte[3]+ byte[4]*256)
  // byte 5, n+5 - the save data
  
  unsigned short i;
  
  // set OK flag
  pDataDest[0] = 0; // 0 for fail, 1 for OK
  
  // check is our game - get data size..
  unsigned short nSize = GetLoadDataSize(pKey);
  if (nSize == 0) {
    // too much data..
    pDataDest[1] = 2;
    return;
  }
  // ok to here...
  unsigned char *pDataEnc = (unsigned char*) malloc(nSize*sizeof(unsigned char));
  
  for (i = 0; i < nSize; ++i) {
    pDataEnc[i] = LoadByte(5+i);
  }
  pDataDest[0] = 1;
  // sort the data back into its proper format, stuff it in pDataDest
  UnencryptSaveData(pDataEnc, nSize, &pDataDest[1]);
  
  // all done!
  free(pDataEnc);

}
#endif

unsigned char LoadByte(unsigned short nOffset) {
  return *(unsigned char *)(SRAM + nOffset);
}

#if 0
void SaveData(unsigned char * pDataSrc, unsigned short nDataLength, char* pKey) {
  // the save structure is:
  // byte 0, 1, 2 - 3 byte Key, e.g. "CYC" for cyclone
  // byte 3, n - the save data - size is stored in bytes 4,5 and 6
  
  unsigned short i; // looping var
  
  // 1st save the key from our game...
  for (i = 0; i < 3; i++) {
    SaveByte(i, pKey[i]);
  }
  
  // encrypt the source data...  
  unsigned char * pDataEnc = (unsigned char*)malloc( nDataLength*sizeof(unsigned char));
  EncryptSaveData(pDataSrc, nDataLength, pDataEnc);
  
  // save the size of the data...
  unsigned char nSizeLo = nDataLength& 0xff;
  unsigned char nSizeHi = (nDataLength>>8) & 0xff;
  SaveByte(3, nSizeLo); SaveByte(4, nSizeHi);
  
  // save the actual data
  for (i = 0; i < nDataLength; i++) {
    SaveByte(5+i, pDataEnc[i]);
  }
  // all done, free the encrypted array and return
  free(pDataEnc);
  
}
#endif

void SaveByte(unsigned short nOffset, unsigned char cValue) {
  //Save the value at the given offset
  *(unsigned char *)(SRAM + nOffset) = cValue;
}



#if 0
void EncryptSaveData(unsigned char* pDataSrc, unsigned short nDataLength, unsigned char* pDataDest) {
  // encryption is simple, add the "key" value at nKeyPos to 
  // the byte at position [i] in the pDataSrc array
  // output this new byte to the pDataDest array at [nDataLength-i]!
  // increment position, when position reaches MAX_KEY_LENGTH, set to 0
  
  // decryption is the same, just subtract the key at the value
  unsigned char nKeyPos = 0;
  unsigned short i;
  for (i = 0; i < nDataLength; ++i) {
    pDataDest[nDataLength-1-i] = pDataSrc[i] + cEncryptionKey[nKeyPos];
    nKeyPos++;
    if (nKeyPos == MAX_KEY_LENGTH)
      nKeyPos = 0;
  }
  
  
}

void UnencryptSaveData(unsigned char* pDataSrc, unsigned short nDataLength, unsigned char* pDataDest) {
  unsigned char nKeyPos = 0;
  signed short i;
  for (i = nDataLength; i > 0; --i) {
    pDataDest[nDataLength-i] = pDataSrc[i-2] - cEncryptionKey[nKeyPos];
    
    nKeyPos++;
    if (nKeyPos == MAX_KEY_LENGTH)
      nKeyPos = 0;
  }
  
}
#endif
  

