#include "gba.h"
#include "backgrounds.h"
#include "screenmode.h"

void wait_vsync_int(void) {
#ifdef __thumb__
    asm volatile("swi 0x5");
#else
    asm volatile("swi 0x5<<16");
#endif
}

void screen_fade(int level) 
{
  if (level > 16)
    level = 16;
  if (level < 0)
    level = 0;
  REG_BLDMOD = BLDMOD_FIRST_BG1 | BLDMOD_FIRST_BG0 | BLDMOD_FIRST_BG2|
    BLDMOD_FIRST_OBJ | BLDMOD_SECOND_BD | BLDMOD_MODE(3);
  REG_COLEY = level;
}

void screen_fade_out()
{
  int loop;
  for (loop = 0; loop < 17; loop++) {
    wait_vsync_int();
    screen_fade(loop);
  }
}
void screen_fade_in()
{
  int loop;
  for (loop = 0; loop < 17; loop++) {
    wait_vsync_int();
    screen_fade(16-loop);
  }
}

void screen_white(int level) 
{
  if (level > 16)
    level = 16;
  if (level < 0)
    level = 0;
  REG_BLDMOD = BLDMOD_FIRST_BD 
     | BLDMOD_MODE(2);
  REG_COLEY = level;
}

void screen_white_out()
{
  int loop;
  for (loop = 0; loop < 17; loop++) {
    wait_vsync_int();
    screen_white(loop);
  }
}
void screen_white_in()
{
  int loop;
  for (loop = 0; loop < 17; loop++) {
    wait_vsync_int();
    screen_white(16-loop);
  }
}


// clears out video memory, which may have been set already by the boot menu
// should be one of the first things that we call
void clear_vram(void) {
  unsigned int i;
  // fade down
  // screen_fade(16);

  unsigned int * vram = (unsigned int*)VideoBuffer;
  for (i = 0; i < 0x3100/8; i++) {
    *vram++ = 0;
    *vram++ = 0;
    *vram++ = 0;
    *vram++ = 0;
  }

  for (i = 0; i < 256; i++) {
    BGPaletteMem[i] = 0;
    OBJPaletteMem[i] = 0;
  }
  // fade up again
  //screen_fade(0);
}
